/*	This program is released under European Union Public License.
 *  for details please refer to the file LICENSE
 *   
 *  Disclaimer: the program comes "as is" and with absolutely no warranty 
 *  
 *  © 2010 TCM
 */

package gui.filechooser;
import gui.filelist.FileList;
import gui.filetree.FileTree;
import gui.okcancelpanel.OKCancelPanel;
import gui.okcancelpanel.OKCondition;
import gui.util.combobox.VComboBox;
import gui.util.combobox.VComboBoxModel;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Window;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.IOException;
import java.nio.file.Path;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import util.exception.CancelException;

public class FilesChooser extends JDialog {
	private static final long serialVersionUID = 1L;
	private OKCancelPanel buttonPanel; 
	private FileTree fileTree;
	private FileList fileList;
	private JSplitPane splitPane;
	private JTextField txtPath;
	
	private VComboBox cbFilter;
	private VComboBoxModel<FilesChooserFilter> cbFilterModel;
	
	private FilesChooserFilter currentFilter;
	private List<FilesChooserFilter> filters;
	public static final int OK_OPTION=JOptionPane.OK_OPTION;
	public static final int CANCEL_OPTION=JOptionPane.CANCEL_OPTION;
	
	private JCheckBox chkAppend;
	
	public FilesChooser(Window owner,String title,boolean multiselect, String okCaption,String cancelCaption) {
		super((Frame)null,title,true);
		setSize(480, 360);
		Container contentPane=getContentPane();
		contentPane.setLayout(new BorderLayout());
		filters=new LinkedList<FilesChooserFilter>();
		
		buttonPanel=new OKCancelPanel(this,	 okCaption, cancelCaption);
		
		cbFilterModel=new VComboBoxModel<FilesChooserFilter>();
		cbFilter=new VComboBox(cbFilterModel);
		fileList=new FileList(multiselect,"Name","Size","Date");
		
		fileList.addSelectionListener(new ListSelectionListener() {
			
			@Override
			public void valueChanged(ListSelectionEvent e) {
				
				Path [] files=fileList.getSelectedFiles();
				if (files!=null) {
					int k=files.length-1;
					
					StringBuilder sb=new StringBuilder();
					for (int i=0;i<k;i++) {
						sb.append(files[i].getFileName());
						sb.append("|");
					}
					sb.append(files[k].getFileName());
					txtPath.setText(sb.toString());
				}
			}
		});
		
		cbFilter.addItemListener(new ItemListener() {

			@Override
			public void itemStateChanged(ItemEvent e) {
				if (cbFilterModel.getSelectedValue()!=null) {
					try {
						refreshPathlist() ;
					} catch (IOException e1) {
						e1.printStackTrace();
					}
				}
			}
		});
		
		fileList.addDoubleClickActionListener(buttonPanel.getOkAction());
		fileTree=new FileTree(fileList);
		JPanel p=new JPanel();
		p.setLayout(new BorderLayout());
		p.add(fileList,BorderLayout.CENTER);
		p.add(cbFilter,BorderLayout.SOUTH);
		splitPane=new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, fileTree, p);
		splitPane.setDividerLocation(200);
		txtPath=new JTextField();
		
		Dimension d=new Dimension(480, 24);
		txtPath.setPreferredSize(d);
		contentPane.add(txtPath,BorderLayout.NORTH);
		contentPane.add(splitPane,BorderLayout.CENTER);
		contentPane.add(buttonPanel,BorderLayout.SOUTH);
		
		getRootPane().setDefaultButton(buttonPanel.getOkButton());
	}
	
	private void setDir(Path dir) {
		fileTree.expandFileTree(dir);
	}
	
	public void setChooseFilters(FilesChooserFilter... filters) {
		this.filters.clear();
		fileList.removeAllFilters();
		cbFilterModel.removeAllElements();		
		for (FilesChooserFilter f:filters) {
			fileList.addFileFilter(f.getFilter());
			this.filters.add(f);
			cbFilterModel.addElement(f.getDisplayName(), f);
		}
	}
	
	private void setChooseFilter(FilesChooserFilter defaultFilter) throws IOException {
		if (defaultFilter==null) {
			if (filters.size()>0) {
				
				cbFilterModel.setSelectedItemByValue(filters.get(0));
				refreshPathlist() ;
			}
		} else {
		
			cbFilterModel.setSelectedItemByValue(defaultFilter);
		}
	}
	
	private void refreshPathlist() throws IOException {
		currentFilter=cbFilterModel.getSelectedValue();
		fileList.removeAllFilters();
		fileList.addFileFilter(currentFilter.getFilter());
		fileList.refresh();
	}
	
	public int showOpenFilesDialog(Path dir) throws IOException, CancelException {
		return showOpenFilesDialog(dir,null);
	}
	
	public int showOpenFilesDialog(Path dir,FilesChooserFilter defaultChooseFilter) throws IOException, CancelException {
		setChooseFilter(defaultChooseFilter);
		buttonPanel.setOKCondition(new OKCondition() {
			
			@Override
			public boolean cancel() {
				return fileList.getSelectedFiles()==null;
			}
		});
		if (dir!=null) setDir(dir);
		return showOpenFilesDialog();
	}
	
	
	private int showOpenFilesDialog() throws CancelException {
		setVisible(true);
		return buttonPanel.getRetVal();
	}
	
	public int showSavePathDialog(Path dir) throws IOException, CancelException {
		return showSavePathDialog(dir, null);
	}
	
	public int showSavePathDialog(Path dir,FilesChooserFilter defaultChooseFilter) throws IOException, CancelException {
		setChooseFilter(defaultChooseFilter);
		buttonPanel.setOKCondition(null);
		setDir(dir);

		txtPath.setVisible(true);
		txtPath.setText("");
		setVisible(true);
		String fn=txtPath.getText();
		if (currentFilter!=null ) {
			String ext=currentFilter.getExtension();
			
			if (ext!=null && !fn.endsWith(ext))
					txtPath.setText(fn+ext);
		}
		return buttonPanel.getRetVal();
	}
	
	public Path getFileTreePath() {
		return fileTree.getSelectedDir(); 
	}
	
	public Path getSelectedPath() {
		return fileTree.getSelectedDir().resolve(txtPath.getText());
	}
	
	public Path[] getSelectedPaths() {
		String[] filenames=txtPath.getText().split("\\|");
		Path[] files=new Path[filenames.length];

		for (int i=0;i<filenames.length;i++) {
			files[i]=fileTree.getSelectedDir().resolve(filenames[i]);
		}
		
		return fileList.getSelectedFiles();
	}
	
	public void setShowAppendOption(String text,boolean preSelect){
		chkAppend=new JCheckBox(text,preSelect);
		buttonPanel.add(chkAppend);
	}
	
	public boolean isAppendMode() {
		return chkAppend!=null?chkAppend.isSelected():false;
	}

	public Path getDir(){
		return fileTree.getSelectedDir();
	}
}
