/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package gui.layout;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager2;
import java.util.Hashtable;

public class PercentageLayout implements LayoutManager2 {
	private Hashtable<Component, PercentageLayoutConstraints> compConstraints;
	private Dimension minSize;
	
	public PercentageLayout() {
		this(new Dimension());
	}

	public PercentageLayout(Dimension minsize) {
		compConstraints = new Hashtable<Component, PercentageLayoutConstraints>();
		setMinSize(minsize);
	}
	
	public static PercentageLayoutConstraints constraints(int x,int y,int width,int height) {
		return new PercentageLayoutConstraints(x, y, width, height);
	}
	
	public static PercentageLayoutConstraints constraints(int x,int y,int width,int height,boolean absX,boolean absY,boolean absWidth,boolean absHeight) {
		return new PercentageLayoutConstraints(x, y, width, height, absX, absY, absWidth, absHeight);
	}
	
	public static PercentageLayoutConstraints constraints(int x,int y) {
		return new PercentageLayoutConstraints(x,y,100-x*2,100-y*2);
	}
	
	@Override
	public void layoutContainer(Container target) {
		synchronized (target.getTreeLock()) {
			Insets border=target.getInsets();
			int width = target.getWidth() - border.left-border.right;
			int height = target.getHeight() - border.top-border.bottom;
				
			for (Component c : target.getComponents()) {
				c.invalidate();
				PercentageLayoutConstraints con=compConstraints.get(c);
				if (con==null) return;
				
				int[] bounds = con.getBounds();
				boolean[] absolute=con.getAbsolute();
				
				if (bounds != null && bounds.length>0) {
					
					int x,y,w,h;
					
					if (absolute[0]) {
						x=bounds[0];
					} else {
						x=bounds[0] *  width / 100;
					}
					if (absolute[1]) {
						y=bounds[1];
					} else {
						y=bounds[1] * height / 100;
					}
					if (absolute[2]) {
						w=bounds[2];
					} else {
						w=bounds[2] * width / 100;
					}
					if (absolute[3]) {
						h=bounds[3];
					} else {
						h=bounds[3] * height / 100;
					}
					
					c.setBounds(x+border.left, y+border.top, w, h);
					c.validate();
				}
			}
		}
	}

	@Override
	public void addLayoutComponent(Component comp, Object constraints) {
		synchronized (comp.getTreeLock()) {
			PercentageLayoutConstraints constr = (PercentageLayoutConstraints) constraints;
			
			if (constr!=null) compConstraints.put(comp, constr);
		}
	}

	@Override
	public float getLayoutAlignmentX(Container target) {
		return 0.5f;
	}

	@Override
	public float getLayoutAlignmentY(Container target) {
		return 0.5f;
	}

	@Override
	public void invalidateLayout(Container target) {
	}

	@Override
	public Dimension maximumLayoutSize(Container target) {
		return target.getSize();
	}

	@Override
	public void addLayoutComponent(String name, Component comp) {
		throw new UnsupportedOperationException();
	}

	@Override
	public Dimension minimumLayoutSize(Container parent) {
		return minSize;
	}

	@Override
	public Dimension preferredLayoutSize(Container parent) {
		return parent.getSize();
	}

	@Override
	public void removeLayoutComponent(Component comp) {
		synchronized (comp.getTreeLock()) {
			compConstraints.remove(comp);
		}
	}
	
	public void setMinSize(Dimension minSize) {
		this.minSize = minSize;
	}
	
	private static class PercentageLayoutConstraints {
		private int[] bounds;
		private boolean[] absolute;
		
		public PercentageLayoutConstraints(int x,int y,int width,int height) {
			this(x, y, width, height, false, false, false, false);
		}
		
		public PercentageLayoutConstraints(int x,int y,int width,int height,boolean absX,boolean absY,boolean absWidth,boolean absHeight) {
			bounds= new int[] {x,y,width,height};
			absolute= new boolean[] {absX,absY,absWidth,absHeight};
		}
		
		public int[] getBounds() {
			return bounds;
		}
		
		public boolean[] getAbsolute() {
			return absolute;
		}
	}
}